@@ -5,9 +5,8 @@ namespace apexl\Io\modules\user\controllers;
 use apexl\hashing\Hash;
 use apexl\Io\exceptions\RecordNotFoundException;
 use apexl\Io\includes\Controller;
-use apexl\Io\includes\Hook;
-use apexl\Io\includes\Routes;
-use apexl\Io\includes\System;
+use apexl\Io\includes\HookManager;
+use apexl\Io\includes\RouteManager;
 use apexl\Io\modules\email\services\templateService;
 use apexl\Io\modules\user\entities\refreshTokenEntity;
 use apexl\Io\modules\user\entities\sessionEntity;
@@ -18,30 +17,38 @@ use apexl\Io\modules\user\exceptions\ClientCredentialsValidationException;
 use apexl\Io\modules\user\exceptions\RefreshTokenExpiredException;
 use apexl\Io\modules\user\exceptions\RefreshTokenUserMismatchException;
 use apexl\Io\modules\user\services\currentUser;
+use apexl\Io\modules\user\services\CurrentUserFactory;
 use apexl\Io\modules\user\services\RefreshTokenSessionIdFinder;
 use apexl\Io\modules\user\services\userTools;
+use apexl\Io\services\HttpPaths;
+use apexl\Io\services\Mailer;
+use apexl\Io\services\Output;
+use apexl\Vault\exceptions\ExecutionException;
 use DateTimeImmutable;
 use Exception;
-use Psr\Http\Message\ResponseInterface as Response;
-use Psr\Http\Message\ServerRequestInterface as Request;
+use Psr\Http\Message\ResponseInterface;
+use Psr\Http\Message\ServerRequestInterface;
 
-class userController extends Controller
+final readonly class userController extends Controller
 {
     protected userEntity $currentUser;
 
     public function __construct(
-        currentUser $currentUser,
         protected userTools $userTools,
-        readonly private RefreshTokenSessionIdFinder $sessionIdFinder
+        private RefreshTokenSessionIdFinder $sessionIdFinder,
+        private HttpPaths $path,
+        private Mailer $mailer,
+        currentUserFactory $currentUserFactory,
+        Output $output,
     ) {
-        parent::__construct();
-        $this->currentUser = $currentUser::getCurrentUser();
+        parent::__construct($output);
+        $this->currentUser = $currentUserFactory->get();
     }
 
     /**
      * @throws Exception
      */
-    public function login(Request $request, Response $response): Response
+    public function login(HookManager $hookManager, ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
     {
         $body = (object) $request->getParsedBody();
         if (isset($body->grant_type) && $body->grant_type === 'refresh_token') {
@@ -53,14 +60,14 @@ class userController extends Controller
 
         //basic validation
         if (empty($body) || !isset($body->username) || !isset($body->password)) {
-            $this->output::addMessage(
+            $this->output->addMessage(
                 'user.login.validation',
                 'error',
                 "Please provide a username and a password."
             );
-            $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+            $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-            return System::asJson($response, [], 400);
+            return $this->json($response, [], 400);
         }
 
         //first, grab the user data.
@@ -73,38 +80,26 @@ class userController extends Controller
             $match = $hash->matchHash($body->password, $user->password, $user->salt);
             //if we match, complete the login process
 
-            $match = Hook::processHook('userLogin', $match, $user, $request);
+            $match = $hookManager->processHook('userLogin', $match, $user, $request);
             if ($match) {
-                [$authToken, $refreshToken] = $this->userTools->startLoggedInSession($user);
-                //allow others to act on this response. Hook is keyed by route name.
-                $this->output::addResponse(
-                    $request,
-                    [
-                        'access_token' => $authToken,
-                        'refresh_token' => $refreshToken,
-                        'expires_in' => currentUser::getTokenExpiryFromNow($authToken),
-                    ]
-                );
-
+                $this->loginPostAuth($user, $request);
                 if ($redirect = ((object) $request->getParsedBody())->redirect ?? null) {
-                    $this->output::addRedirect('user.login.redirect', $redirect);
+                    $this->output->addRedirect('user.login.redirect', $redirect);
                 }
-
-
-                return System::asJson($response);
+                return $this->json($response);
             }
         }
         //if we end up here, then the login attempt failed, so return a 400
-        $this->output::addMessage('user.login.validation', 'error', 'The email address and password don\'t match');
-        $this->output::addResponse($request, [], false); // added so we can hook into this elsewhere.
+        $this->output->addMessage('user.login.validation', 'error', 'The email address and password don\'t match');
+        $this->output->addResponse($request, [], false); // added so we can hook into this elsewhere.
 
-        return System::asJson($response, [], 400);
+        return $this->json($response, [], 400);
     }
 
     public function refresh(
-        Request $request,
-        Response $response,
-    ): Response {
+        ServerRequestInterface $request,
+        ResponseInterface $response,
+    ): ResponseInterface {
         $body = (object) $request->getParsedBody();
 
         try {
@@ -122,16 +117,16 @@ class userController extends Controller
                     $authToken = currentUser::createJWT(
                         $user,
                         $sessionId,
-                        $this->config->app->jwt->secret_key,
-                        $this->config->app->jwt->algorithm,
-                        ($this->config->app->jwt->lifetime ?? 3600)
+                        config('auth.jwt.secret_key'),
+                        config('auth.jwt.algorithm'),
+                        config('auth.jwt.lifetime')
                     );
                     $refreshTokenEntity->expiry = refreshTokenEntity::newExpiryFrom();
                     $refreshTokenEntity->store();
 
-                    $this->output::addResponse($request, ['access_token' => $authToken]);
+                    $this->output->addResponse($request, ['access_token' => $authToken]);
 
-                    return System::asJson($response);
+                    return $this->json($response);
                 }
             }
         } catch (
@@ -140,103 +135,169 @@ class userController extends Controller
         RefreshTokenExpiredException
         $exception
         ) {
-            return System::asJson(
+            return $this->json(
                 $response,
                 ['error' => $exception->getMessage()],
                 401,
             );
         } catch (RecordNotFoundException $exception) {
-            return System::asJson(
+            return $this->json(
                 $response,
                 ['error' => $exception->getMessage()],
                 404,
             );
         } catch (Exception $exception) {
-            return System::asJson(
+            return $this->json(
                 $response,
                 ['error' => $exception->getMessage()],
                 500,
             );
         }
 
-        return System::asJson($response, [], 403);
+        return $this->json($response, [], 403);
+    }
+
+    /**
+     * Allows other processes to force a user login without requiring a password. (Login by URL, Switch User etc)
+     * @throws Exception
+     */
+    protected function loginPostAuth($user, $request): void
+    {
+        [$authToken, $refreshToken] = $this->userTools->startLoggedInSession($user);
+        //allow others to act on this response. Hook is keyed by route name.
+        $this->output->addResponse(
+            $request,
+            [
+                'access_token' => $authToken,
+                'refresh_token' => $refreshToken,
+                'expires_in' => currentUser::getTokenExpiryFromNow($authToken),
+            ]
+        );
+    }
+
+    /**
+     * @throws ExecutionException
+     * @throws Exception
+     */
+    public function loginByKey(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
+    {
+        //is this allowed?
+        if (config('auth.enableQueryStringLogin')) {
+            $uid = $request->getAttribute('user'); // set in an earlier middleware
+            if ($uid != 0) {
+                //already logged in? Return with a redirect to the home page.
+                if ($redirect = ((object) $request->getParsedBody())->redirect ?? '/') {
+                    $this->output->addRedirect('user.login.redirect', $redirect);
+                }
+                //return without doing anything.
+                return $this->json($response);
+            }
+
+            $body = (object) $request->getParsedBody();
+            if (!empty($body) && isset($body->nfc_key)) {
+
+                //we're not logged in, so lets check if we have the appropriate query string, and a valid key.
+                $user = new userEntity();
+                $user->getUserByNfcKey($body->nfc_key);
+
+                //make sure we have a valid record and we're an active user.
+                if ($user->email !== null && $user->active == 1) {
+                    //login
+                    $this->loginPostAuth($user, $request);
+                    if ($redirect = $body->redirect ?? '/') {
+                        $this->output->addRedirect('user.login.redirect', $redirect);
+                    }
+                    return $this->json($response);
+                }
+            }
+            $this->output->addMessage(
+                'user.login.validation',
+                'error',
+                'The provided key doesn\'t match, or the user is locked.'
+            );
+            $this->output->addResponse($request, [], false); // added so we can hook into this elsewhere.
+
+            return $this->json($response, [], 400);
+        }
+
+        return $this->json($response, ['error' => 'Logging in by Key is not allowed on this system'], 403);
     }
 
     /**
      * @throws Exception
      */
-    public function logout(Request $request, Response $response): Response
+    public function logout(ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
     {
         [$token] = currentUser::authenticateJWT(
             $request,
-            $this->config->app->jwt->secret_key,
-            $this->config->app->jwt->algorithm
+            config('auth.jwt.secret_key'),
+            config('auth.jwt.algorithm'),
         );
         if ($token) {
             $session = new sessionEntity();
             $session->load($token->sessionId);
-            if (property_exists($session, 'sessionId') && $session->sessionId !== null) {
+            if ($session->sessionId !== null) {
                 $session->active = false;
                 $session->ended = new DateTimeImmutable();
                 $session->store();
             }
 
-            $this->output::addResponse($request); //added so we can hook into this elsewhere.
+            $this->output->addResponse($request); //added so we can hook into this elsewhere.
 
-            return System::asJson($response);
+            return $this->json($response);
         }
 
-        $this->output::addMessage('user.logout.validation', 'error', 'The user is not logged in');
-        $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+        $this->output->addMessage('user.logout.validation', 'error', 'The user is not logged in');
+        $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-        return System::asJson($response, [], 403);
+        return $this->json($response, [], 403);
     }
 
     /**
      * @throws Exception
      */
-    public function register(Request $request, Response $response, array $args = []): Response
+    public function register(HookManager $hookManager, ServerRequestInterface $request, ResponseInterface $response, array $args = []): ResponseInterface
     {
         $body = $request->getParsedBody();
         $user = new userEntity();
         if ($id = $args['id'] ?? false) {
             //Permissions check
             if ($this->currentUser->id !== $id && !$this->currentUser->isAllowed(User::UPDATE)) {
-                $this->output::addMessage(
-                    $this->path->getRouteName($request).'.validation',
+                $this->output->addMessage(
+                    $this->path->getRouteName($request) . '.validation',
                     'error',
                     'The UpdateUsers permission is required to do this.'
                 );
-                $this->output::addResponse($request, [], false);
+                $this->output->addResponse($request, [], false);
 
                 //added so we can hook into this elsewhere.
-                return System::asJson($response, [], 403);
+                return $this->json($response, [], 403);
             }
             if ($this->currentUser->id === $id && (!$this->currentUser->isAllowed(User::UPDATE) &&
                     !$this->currentUser->isAllowed(User::UPDATE_SELF))) {
-                $this->output::addMessage(
-                    $this->path->getRouteName($request).'.validation',
+                $this->output->addMessage(
+                    $this->path->getRouteName($request) . '.validation',
                     'error',
                     'The UpdateSelf permission is required to do this.'
                 );
-                $this->output::addResponse($request, [], false);
+                $this->output->addResponse($request, [], false);
 
                 //added so we can hook into this elsewhere.
-                return System::asJson($response, [], 403);
+                return $this->json($response, [], 403);
             }
             $user->load($id);
             //check we have all the data we need, make sure password is not required as this is an update.
             $user->setRequiredData(['email', 'first_name', 'last_name']);
         } else {
             if (!$this->currentUser->isAllowed(User::CAN_REGISTER) && !$this->currentUser->isAllowed(User::CREATE)) {
-                $this->output::addMessage(
-                    $this->path->getRouteName($request).'.validation',
+                $this->output->addMessage(
+                    $this->path->getRouteName($request) . '.validation',
                     'error',
                     'The CreateUsers permission is required to do this.'
                 );
-                $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+                $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-                return System::asJson($response, [], 403);
+                return $this->json($response, [], 403);
             }
             //does this user already exist?
             $userExisting = new userEntity();
@@ -246,32 +307,32 @@ class userController extends Controller
             }
 
             if (!empty($user->email)) {
-                $this->output::addMessage(
-                    $this->path->getRouteName($request).'.validation',
+                $this->output->addMessage(
+                    $this->path->getRouteName($request) . '.validation',
                     'error',
                     'A user with that email address already exists.'
                 );
-                $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+                $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-                return System::asJson($response, [], 400);
+                return $this->json($response, [], 400);
             }
         }
 
         //check if passwords match, if we're providing them
         if ($body['password'] ?? false) {
             if ($body['password'] != $body['confirm_password']) {
-                $this->output::addMessage(
-                    $this->path->getRouteName($request).'.validation',
+                $this->output->addMessage(
+                    $this->path->getRouteName($request) . '.validation',
                     'error',
                     'Passwords don\'t match'
                 );
-                $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+                $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-                return System::asJson($response, [], 400);
+                return $this->json($response, [], 400);
             }
             $passwordHash = '';
             $passwordSalt = '';
-            if (isset($body['password']) && !empty($body['password'])) {
+            if (!empty($body['password'])) {
                 $hash = new Hash();
                 $hashData = $hash->hashString($body['password']);
                 $passwordHash = $hashData->hash;
@@ -309,58 +370,58 @@ class userController extends Controller
         $user->roles = serialize([]);
 
         //Hook user pre save - allow other modules to act on the user save process, before storage.
-        $user = Hook::processHook('userPreSave', $user, $request);
+        $user = $hookManager->processHook('userPreSave', $user, $request);
 
         if ($user->isValid()) {
 
             $user->store();
 
             //Hook user post save - allow other modules to act on the user save process, after storage.
-            $user = Hook::processHook('userPostSave', $user, $request);
+            $user = $hookManager->processHook('userPostSave', $user, $request);
 
-        foreach ($body as $name => $value) {
-            if ($this->currentUser->isAllowed(Role::SET)) {
-                if (str_starts_with((string) $name, 'role_')) {
-                    //Check we have permission to set user roles.
-                    if ($value === 'true' || (int)$value !== 0) { //Ignore any FALSE/0/blank string values
-                        $user->addRole(str_replace('role_', '', (string) $name));
+            foreach ($body as $name => $value) {
+                if ($this->currentUser->isAllowed(Role::SET)) {
+                    if (str_starts_with((string) $name, 'role_')) {
+                        //Check we have permission to set user roles.
+                        if ($value === 'true' || (int) $value !== 0) { //Ignore any FALSE/0/blank string values
+                            $user->addRole(str_replace('role_', '', (string) $name));
+                        }
+                    } else {
+                        $user->$name = $value;
                     }
+                } elseif (strpos((string) $name, 'role_')) {
+                    //dont allow users without permission to set roles. Just ignore them and move on.
+                    continue;
                 } else {
                     $user->$name = $value;
                 }
-            } elseif (strpos((string) $name, 'role_')) {
-                //dont allow users without permission to set roles. Just ignore them and move on.
-                continue;
-            } else {
-                $user->$name = $value;
             }
-        }
 
-            $this->output::addMetadata($this->path->getRouteName($request).'.complete', 'entityId', $user->id);
-            $this->output::addMessage(
-                $this->path->getRouteName($request).'.complete',
+            $this->output->addMetadata($this->path->getRouteName($request) . '.complete', 'entityId', $user->id);
+            $this->output->addMessage(
+                $this->path->getRouteName($request) . '.complete',
                 'success',
-                $user->getNiceName().' '.($id ? 'updated' : 'created')
+                $user->getNiceName() . ' ' . ($id ? 'updated' : 'created')
             );
-            $this->output::addResponse($request, $user->getData()); //added so we can hook into this elsewhere.
+            $this->output->addResponse($request, $user->getData()); //added so we can hook into this elsewhere.
 
-            return System::asJson($response);
+            return $this->json($response);
         }
-        $this->output::addMessage(
-            $this->path->getRouteName($request).'.validation',
+        $this->output->addMessage(
+            $this->path->getRouteName($request) . '.validation',
             'error',
             'Not all of the data provided is valid'
         );
-        $this->output::addResponse(
+        $this->output->addResponse(
             $request,
             ['missingFields' => $user->getMissingData()],
             false
         ); //added so we can hook into this elsewhere.
 
-        return System::asJson($response, [], 400);
+        return $this->json($response, [], 400);
     }
 
-    public function forgotPassword(Request $request, Response $response): Response
+    public function forgotPassword(RouteManager $routeManager, ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
     {
         $body = (object) $request->getParsedBody();
         //first, grab the user data.
@@ -374,34 +435,38 @@ class userController extends Controller
             $randomHash = str_replace('/', '@', $hash->generateRandomHash());
             //write the hash to the database
             $user->newForgottenPasswordLink($randomHash);
-            //now we send an email to the user.
-            $this->forgottenPasswordEmail($user->first_name, $user->email, $randomHash);
+            //now we send email to the user.
+            $this->forgottenPasswordEmail(
+                $user->first_name,
+                $user->email,
+                $routeManager->getRoutePattern('user.display.reset-password', ['hash' => $randomHash])
+            );
             $this->output->addMetadata(
                 'user.forgot-password.redirect',
                 'redirect',
-                Routes::getRoutePattern('user.display.forgot-password.check-email')
+                $routeManager->getRoutePattern('user.display.forgot-password.check-email')
             );
         } else {
-            $this->output::addMessage('user.forgot-password', 'error', 'User account does not exist');
+            $this->output->addMessage('user.forgot-password', 'error', 'User account does not exist');
         }
-        $this->output::addResponse($request); //added so we can hook into this elsewhere.
+        $this->output->addResponse($request); //added so we can hook into this elsewhere.
 
-        return System::asJson($response);
+        return $this->json($response);
     }
 
-    protected function forgottenPasswordEmail($firstName, $email, $string): void
+    protected function forgottenPasswordEmail(string $firstName, string $email, string $link): void
     {
         /**
          * @TODO Allow for database config here.
          * @TODO Move all this to the central mailer system, make an easy to use mailer function
          */
-        $from = $this->config->app->site->email_address ?? 'no-reply@localhost.com';
-        $fromName = $this->config->app->site->name ?? 'localhost';
-        $frontEndDomain = $this->config->app->site->frontend_domain ?? 'localhost';
+        $from = config('app.site.email_address') ?? 'no-reply@localhost.com';
+        $fromName = config('app.site.name') ?? 'localhost';
+        $frontEndDomain = config('app.site.frontend_domain') ?? 'localhost';
 
         /** @TODO Move to config. * */
         //$link = rtrim((string) $frontEndDomain, '/').'/api/v1/user/data/forgot-password/h/'.$string;
-        $link = rtrim($frontEndDomain, '/').'/'.ltrim('/'.Routes::getRoutePattern('user.display.reset-password', ['hash' => $string]), '/');
+        $link = rtrim($frontEndDomain, '/') . '/' . ltrim('/' . $link, '/');
 
         try {
             $this->mailer->setFrom($from, $fromName);
@@ -423,13 +488,17 @@ class userController extends Controller
         }
     }
 
-    public function resetPassword(Request $request, Response $response, $args)
+    public function resetPassword(RouteManager $routeManager, ServerRequestInterface $request, ResponseInterface $response, $args): ResponseInterface
     {
-        $body = (object)$request->getParsedBody();
+        $body = (object) $request->getParsedBody();
         if ($body->password != $body->confirm_password) {
-            $this->output::addMessage($this->path->getRouteName($request).'.validation', 'error', 'Passwords don\'t match');
-            $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
-            return System::asJson($response, [], 400);
+            $this->output->addMessage(
+                $this->path->getRouteName($request) . '.validation',
+                'error',
+                'Passwords don\'t match'
+            );
+            $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
+            return $this->json($response, [], 400);
         }
         $isReset = false;
 
@@ -440,8 +509,6 @@ class userController extends Controller
             $user->getUserByEmail($match['email']);
             if (isset($user->id) && $user->id > 0 && isset($body->password) && !empty($body->password)) {
 
-                $passwordHash = '';
-                $passwordSalt = '';
                 $hash = new Hash();
                 $hashData = $hash->hashString($body->password);
                 $passwordHash = $hashData->hash;
@@ -452,17 +519,25 @@ class userController extends Controller
                 $user->salt = $passwordSalt;
                 $user->store();
 
-                $this->output->addMetadata('user.login.validation.redirect', 'redirect', Routes::getRoutePattern('user.display.reset-password-success'));
-                return System::asJson($response, ['success' => true]);
+                $this->output->addMetadata(
+                    'user.login.validation.redirect',
+                    'redirect',
+                    $routeManager->getRoutePattern('user.display.reset-password-success')
+                );
+                return $this->json($response, ['success' => true]);
             }
         }
 
-        $this->output::addMessage($this->path->getRouteName($request).'.validation', 'error', 'We were unable to reset your password');
-        return System::asJson($response, [], 400);
+        $this->output->addMessage(
+            $this->path->getRouteName($request) . '.validation',
+            'error',
+            'We were unable to reset your password'
+        );
+        return $this->json($response, [], 400);
 
     }
 
-    public function forgotPasswordWithLink(Request $request, Response $response, array $args = []): Response
+    public function forgotPasswordWithLink(ServerRequestInterface $request, ResponseInterface $response, array $args = []): ResponseInterface
     {
         $hash = $args['hash'] ?? false;
         if ($hash) {
@@ -475,20 +550,20 @@ class userController extends Controller
                 if ($user->id ?? false) {
                     //log the user in.
                     [$authToken] = $this->userTools->startLoggedInSession($user);
-                    $this->output::addResponse($request, ['access_token' => $authToken]
+                    $this->output->addResponse($request, ['access_token' => $authToken]
                     ); //added so we can hook into this elsewhere.
 
-                    return System::asJson($response);
+                    return $this->json($response);
                 }
             }
         }
-        $this->output::addMessage(
-            $this->path->getRouteName($request).'.validation',
+        $this->output->addMessage(
+            $this->path->getRouteName($request) . '.validation',
             'error',
             "Invalid, unknown or expired hash"
         );
-        $this->output::addResponse($request, [], false); //added so we can hook into this elsewhere.
+        $this->output->addResponse($request, [], false); //added so we can hook into this elsewhere.
 
-        return System::asJson($response, [], 400);
+        return $this->json($response, [], 400);
     }
 }
